import { spawn } from 'child_process'

function git(opts) {
  return opts.git || 'git'
}

function buildCloneCommand(repo, targetPath, opts) {
  let args = [ 'clone' ]
  const userArgs = opts.args || []

  if (opts.shallow) {
    if (userArgs.indexOf('--depth') >= 0) {
      throw new Error('\'--depth\' cannot be specified when shallow is set to \'true\'')
    }
    args.push('--depth', '1')
  }

  if (opts.progress) {
    args.push('--progress')
  }

  args = args.concat(userArgs)
  args.push('--', repo, targetPath)

  return [ git(opts), args ]
}

function buildCheckoutCommand(ref, opts) {
  return [ git(opts), [ 'checkout', ref ] ]
}

export function clone(repo, targetPath, opts, onSuccess, onError) {
  const [ cmd, args ] = buildCloneCommand(repo, targetPath, opts)
  const proc = spawn(cmd, args)

  if (opts.progress) {
    proc.stderr.on('data', (evt) => {
      const line = evt.toString()
      if (line.match(/Receiving objects:\s+(\d+)%/)) {
        opts.progress({
          phase: 'receivingObjects',
          percent: Number(RegExp.$1)
        })
      } else if (line.match(/Resolving deltas:\s+(\d+)%/)) {
        opts.progress({
          phase: 'resolvingDeltas',
          percent: Number(RegExp.$1)
        })
      }
    })
  }

  proc.on('close', (status) => {
    if (status == 0) {
      if (opts.checkout) {
        _checkout()
      } else {
        onSuccess()
      }
    } else {
      onError(new Error('\'git clone\' failed with status ' + status))
    }
  })

  function _checkout() {
    const [ cmd, args ] = buildCheckoutCommand(opts.checkout, opts)
    const proc = spawn(cmd, args, { cwd: targetPath })
    proc.on('close', function (status) {
      if (status == 0) {
        onSuccess()
      } else {
        onError(new Error('\'git checkout\' failed with status ' + status))
      }
    })
  }
}
